﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO.Ports;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows.Forms;
//using System.Diagnostics;

namespace Terminal
{
	public partial class Form1 : Form
	{
		private SerialPort mySerialPort;
		private string prijateData = string.Empty;
		private mod recMod = mod.ASCI;
		private mod transMod = mod.ASCI;
		private string separator = " ";
		enum mod
		{
			ASCI=0,
			HEX,
			DEC,
			BINARY
		}
		public Form1()
		{
			InitializeComponent();
			this.Text = "Terminal " + "v." + Assembly.GetExecutingAssembly().GetName().Version.ToString();
			this.initCombobox();
			this.enabledCombobox(true);
			this.mySerialPort = new SerialPort();
			this.mySerialPort.DataReceived += new SerialDataReceivedEventHandler(mySerialPort_DataReceived);
		}
		delegate void SetControlValueCallback(Control oControl, string propName, object propValue);
		private void SetControlPropertyValue(Control oControl, string propName, object propValue)
		{
			if (oControl.InvokeRequired)
			{
				SetControlValueCallback d = new SetControlValueCallback(SetControlPropertyValue);
				oControl.Invoke(d, new object[] { oControl, propName, propValue });
			}
			else
			{
				Type t = oControl.GetType();
				PropertyInfo[] props = t.GetProperties();
				foreach (PropertyInfo p in props)
				{
					if (p.Name.ToUpper() == propName.ToUpper())
					{
						p.SetValue(oControl, propValue, null);
					}
				}
			}
		}
		void mySerialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
		{
			try
			{
				byte[] buffer = new byte[0];
				
				while (mySerialPort.BytesToRead > 0)
				{
					buffer = new byte[mySerialPort.BytesToRead];
					this.mySerialPort.Read(buffer, 0, mySerialPort.BytesToRead);
				}

				if (this.recMod == mod.ASCI)
				{
					this.prijateData += Encoding.ASCII.GetString(buffer);
				}
				if (this.recMod == mod.HEX)
				{
					foreach(byte s in buffer)
					{
						this.prijateData += s.ToString("X2") + this.separator;
					}
				}
				if (this.recMod == mod.DEC)
				{
					foreach (byte s in buffer)
					{
						this.prijateData += s.ToString() + this.separator;
					}
				}
				if (this.recMod == mod.BINARY)
				{
					foreach (byte s in buffer)
					{
						this.prijateData += Convert.ToString(s, 2).PadLeft(8,'0') + this.separator;
					}
				}

				// this.prijateData += Encoding.ASCII.GetString(buffer);
				this.SetControlPropertyValue(this.richTextBox_rec, "Text", this.prijateData);
			}
			catch (Exception ex1)
			{
				MessageBox.Show(ex1.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
		
		private void initCombobox()
		{
			//parity
			foreach (string s in Enum.GetNames(typeof(Parity)))
			{
				this.comboBox_parity.Items.Add(s);
			}
			//stop bits
			foreach (string s in Enum.GetNames(typeof(StopBits)))
			{
				this.comboBox_stop.Items.Add(s);
			}
			//data bits
			this.comboBox_data.Items.AddRange(new string[]{"5","6","7","8"});
			//baud rate
			this.comboBox_baud.Items.AddRange(
				new string[] {
					"600",
					"1200",
					"2400",
					"4800",
					"9600",
					"14400",
					"19200",
					"28800",
					"38400",
					"56000",
					"57600",
					"115200",
					"128000",
					"256000", }
			);
			//COM port list
			this.comboBox_ports.Items.AddRange(SerialPort.GetPortNames());

			this.comboBox_ports.SelectedIndex = 0;
			this.comboBox_baud.SelectedIndex = 4;
			this.comboBox_data.SelectedIndex = 3;
			this.comboBox_parity.SelectedIndex = 0;
			this.comboBox_ports.SelectedIndex = 0;
			this.comboBox_rec.SelectedIndex = 0;
			this.comboBox_stop.SelectedIndex = 1;
			this.comboBox_trans.SelectedIndex = 0;
			this.comboBox_sep.SelectedIndex = 0;
		}
		private void enabledCombobox(bool enable)
		{
			this.comboBox_baud.Enabled = enable;
			this.comboBox_data.Enabled = enable;
			this.comboBox_parity.Enabled = enable;
			this.comboBox_ports.Enabled = enable;
			this.comboBox_rec.Enabled = enable;
			this.comboBox_stop.Enabled = enable;
			this.comboBox_trans.Enabled = enable;
			this.button_send.Enabled = !enable;
		}

		private void button1_Click(object sender, EventArgs e)
		{
			try
			{
				if (this.mySerialPort.IsOpen == false)
				{
					this.mySerialPort.PortName = this.comboBox_ports.Text;
					this.mySerialPort.BaudRate = Convert.ToInt32(this.comboBox_baud.Text);
					this.mySerialPort.DataBits = Convert.ToInt32(this.comboBox_data.Text);
					this.mySerialPort.Parity = (Parity)Enum.Parse(typeof(Parity), this.comboBox_parity.Text);
					this.mySerialPort.StopBits = (StopBits)Enum.Parse(typeof(StopBits), this.comboBox_stop.Text);
					this.enabledCombobox(false);

					this.mySerialPort.Open();
					this.button1.Text = "Disconnect";
				}
				else
				{
					this.mySerialPort.Close();
					this.enabledCombobox(true);
					this.button1.Text = "Connect";
				}
			}
			catch (Exception ex1)
			{
				MessageBox.Show(ex1.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void button2_Click(object sender, EventArgs e)
		{
			this.richTextBox_rec.Clear();
			this.prijateData = string.Empty;
		}

		private void button3_Click(object sender, EventArgs e)
		{
			this.richTextBox_trans.Clear();
		}

		private void button5_Click(object sender, EventArgs e)
		{
			try
			{
				if (this.transMod == mod.ASCI)
				{
					this.mySerialPort.WriteLine(this.richTextBox_trans.Text);
				}
				if (this.transMod == mod.HEX)
				{
					byte[] data;
					int i = 0;
					string[] dataString = this.richTextBox_trans.Text.Split(new string[] { this.comboBox_sep.Text }, StringSplitOptions.RemoveEmptyEntries);
					data = new byte[dataString.Length];
					foreach (string d1 in dataString)
					{
						data[i++] = Convert.ToByte(d1, 16);
					}

					this.mySerialPort.Write(data, 0, data.Length);
				}
				if (this.transMod == mod.DEC)
				{
					byte[] data;
					int i = 0;
					string[] dataString = this.richTextBox_trans.Text.Split(new string[] { this.comboBox_sep.Text }, StringSplitOptions.RemoveEmptyEntries);
					data = new byte[dataString.Length];
					foreach (string d1 in dataString)
					{
						data[i++] = Convert.ToByte(d1, 10);
					}

					this.mySerialPort.Write(data, 0, data.Length);
				}
				if (this.transMod == mod.BINARY)
				{
					byte[] data;
					int i = 0;
					string[] dataString = this.richTextBox_trans.Text.Split(new string[] { this.comboBox_sep.Text }, StringSplitOptions.RemoveEmptyEntries);
					data = new byte[dataString.Length];
					foreach (string d1 in dataString)
					{
						data[i++] = Convert.ToByte(d1, 2);
					}

					this.mySerialPort.Write(data, 0, data.Length);
				}
			}
			catch(Exception ex1)
			{
				MessageBox.Show(ex1.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void comboBox_trans_SelectedIndexChanged(object sender, EventArgs e)
		{
			this.comboBox_sep.Enabled = false;
			//trans
			this.separator = this.comboBox_sep.Text;
			if (this.comboBox_trans.Text.CompareTo("ASCI") == 0)
			{
				this.transMod = mod.ASCI;
				
			}
			if (this.comboBox_trans.Text.CompareTo("HEX") == 0)
			{
				this.comboBox_sep.Enabled = true;
				this.transMod = mod.HEX;
			}
			if (this.comboBox_trans.Text.CompareTo("DEC") == 0)
			{
				this.comboBox_sep.Enabled = true;
				this.transMod = mod.DEC;
			}
			if (this.comboBox_trans.Text.CompareTo("BINARY") == 0)
			{
				this.comboBox_sep.Enabled = true;
				this.transMod = mod.BINARY;
			}
			//rec
			if (this.comboBox_rec.Text.CompareTo("ASCI") == 0)
			{
				this.recMod = mod.ASCI;
			}
			if (this.comboBox_rec.Text.CompareTo("HEX") == 0)
			{
				this.comboBox_sep.Enabled = true;
				this.recMod = mod.HEX;
			}
			if (this.comboBox_rec.Text.CompareTo("DEC") == 0)
			{
				this.comboBox_sep.Enabled = true;
				this.recMod = mod.DEC;
			}
			if (this.comboBox_rec.Text.CompareTo("BINARY") == 0)
			{
				this.comboBox_sep.Enabled = true;
				this.recMod = mod.BINARY;
			}
			
		}
		
		void Label1Click(object sender, EventArgs e)
		{
			Process.Start("http://www.projectik.eu");
		}
	}
}
